﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Derbster.DataAccess;

namespace Derbster.Actions
{
    public class GameActionStack : IGameActionStack
    {
        private readonly IDerbsterContext _derbsterContext;

        public int CurrentAction { get; private set; }
        public List<IGameAction> Actions { get; private set; }

        public GameActionStack(IDerbsterContext derbsterContext)
        {
            Contract.Requires(derbsterContext != null);
            _derbsterContext = derbsterContext;
            Actions = new List<IGameAction>();
            CurrentAction = 0;
        }

        public void Clear()
        {
            Actions.Clear();
            CurrentAction = 0;
        }

        public void Push(IGameAction newAction)
        {
            Actions.Add(newAction);
            CurrentAction++;
        }

        public IGameAction Pop()
        {
            var last = Actions.Last();
            Actions.Remove(last);
            return last;
        }

        public void Redo()
        {
            if (Actions.Count == 0 || CurrentAction == Actions.Count) return;

            var action = Actions[CurrentAction];
            action.Do();
            _derbsterContext.SaveChanges();
            CurrentAction++;
        }

        public void Undo()
        {
            if (Actions.Count == 0 || CurrentAction == 0) return;

            var action = Actions[CurrentAction-1];
            action.Undo();
            _derbsterContext.SaveChanges();
            CurrentAction--;
        }
    }
}
