﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Derbster.Domain;

namespace Derbster.Actions
{
    [ContractClass(typeof(JamActionsContract))]
    public interface IJamActions
    {
        void FirstTeamScore(int score);
        void SecondTeamScore(int score);

        void CorrectFirstTeamScore(int pass, int score);
        void CorrectSecondTeamScore(int pass, int score);

        void RemoveFirstTeamScoringPass();
        void RemoveSecondTeamScoringPass();

        void AddJammerToFirstTeamLineup(Skater skater);
        void AddPivotToFirstTeamLineup(Skater skater);
        void AddBlockerToFirstTeamLineup(Skater skater);

        void AddJammerToSecondTeamLineup(Skater skater);
        void AddPivotToSecondTeamLineup(Skater skater);
        void AddBlockerToSecondTeamLineup(Skater skater);

        void CorrectFirstTeamLineup(Skater oldSkater, Skater newSkater);
        void CorrectSecondTeamLineup(Skater oldSkater, Skater newSkater);

        void RemoveFromFirstTeamLineup(Skater skater);
        void RemoveFromSecondTeamLineup(Skater skater);

        void FirstTeamIsLead();
        void FirstTeamLostLead();

        void SecondTeamIsLead();
        void SecondTeamLostLead();

        void JamCalled();

        void FirstTeamStarPassed();
        void SecondTeamStarPassed();

        void RevokeFirstTeamStarPass();
        void RevokeSecondTeamStarPass();
    }

    [ContractClassFor(typeof(IJamActions))]
    public abstract class JamActionsContract : IJamActions
    {
        public void FirstTeamScore(int score)
        {
            Contract.Requires(score >= 0 || score <= 6);
        }

        public void SecondTeamScore(int score)
        {
            Contract.Requires(score >= 0 || score <= 6);
        }

        public void CorrectFirstTeamScore(int pass, int score)
        {
            Contract.Requires(pass > 0);
            Contract.Requires(score >= 0 || score <= 6);
        }

        public void CorrectSecondTeamScore(int pass, int score)
        {
            Contract.Requires(pass > 0);
            Contract.Requires(score >= 0 || score <= 6);
        }

        public void RemoveFirstTeamScoringPass()
        {
            
        }

        public void RemoveSecondTeamScoringPass()
        {
            
        }

        public void AddJammerToFirstTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void AddPivotToFirstTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void AddBlockerToFirstTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void AddJammerToSecondTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void AddPivotToSecondTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void AddBlockerToSecondTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void CorrectFirstTeamLineup(Skater oldSkater, Skater newSkater)
        {
            Contract.Requires(oldSkater != null);
            Contract.Requires(newSkater != null);
        }

        public void CorrectSecondTeamLineup(Skater oldSkater, Skater newSkater)
        {
            Contract.Requires(oldSkater != null);
            Contract.Requires(newSkater != null);
        }

        public void RemoveFromFirstTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void RemoveFromSecondTeamLineup(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void FirstTeamIsLead()
        {
            
        }

        public void FirstTeamLostLead()
        {
            
        }

        public void SecondTeamIsLead()
        {
            
        }

        public void SecondTeamLostLead()
        {
            
        }

        public void JamCalled()
        {
            
        }

        public void FirstTeamStarPassed()
        {
            
        }

        public void SecondTeamStarPassed()
        {
            
        }

        public void RevokeFirstTeamStarPass()
        {
            
        }

        public void RevokeSecondTeamStarPass()
        {
            
        }
    }
}
