﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Derbster.Domain;

namespace Derbster.Actions
{
    [ContractClass(typeof(MatchActionsContract))]
    public interface IMatchActions
    {
        Jam CurrentJam { get; }
        bool JamInProgress { get; }
        bool HasStarted { get; }
        bool HasFinished { get; }

        void StartMatch();
        void EndMatch();

        void StartPeriod();
        void EndPeriod();

        Jam StartJam(Lineup firstLineup, Lineup secondLineup);
        void EndJam();

        Penalty SkaterPenalty(Skater skater, string code);
        void CorrectPenalty(Skater skater, int period, int jamNumber, string code);
        void CorrectLastPenalty(Skater skater, string code);
        void RemoveLastPenalty(Skater skater);
        void RemovePenalty(Penalty penalty);

        BoxEvent SkaterSeated(Skater skater);
        BoxEvent SkaterReleased(Skater skater);

        bool VerifyMatch();
    }

    [ContractClassFor(typeof(IMatchActions))]
    public abstract class MatchActionsContract : IMatchActions
    {
        private const string Codes = "ABCEFGHILMNOPSUXZ";
        
        public Jam CurrentJam { get; private set; }
        public bool JamInProgress { get; private set; }
        public bool HasStarted { get; private set; }
        public bool HasFinished { get; private set; }

        public void StartMatch()
        {
            
        }

        public void EndMatch()
        {
            
        }

        public void StartPeriod()
        {
            
        }

        public void EndPeriod()
        {
            
        }

        public Jam StartJam(Lineup firstLineup, Lineup secondLineup)
        {
            Contract.Requires(firstLineup != null);
            Contract.Requires(firstLineup.Count() > 0);
            Contract.Requires(secondLineup != null);
            Contract.Requires(secondLineup.Count() > 0);
            return null;
        }

        public void EndJam()
        {

        }

        public Penalty SkaterPenalty(Skater skater, string code)
        {
            Contract.Requires(skater != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(code));
            Contract.Requires(Codes.Contains(code));
            return null;
        }

        public void CorrectPenalty(Skater skater, int period, int jamNumber, string code)
        {
            Contract.Requires(skater != null);
            Contract.Requires(period == 1 || period == 2);
            Contract.Requires(jamNumber > 0);
            Contract.Requires(string.IsNullOrWhiteSpace(code));
            Contract.Requires(Codes.Contains(code));
        }

        public void CorrectLastPenalty(Skater skater, string code)
        {
            Contract.Requires(skater != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(code));
            Contract.Requires(Codes.Contains(code));
        }

        public void RemoveLastPenalty(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void RemovePenalty(Penalty penalty)
        {
            Contract.Requires(penalty != null);
        }

        public BoxEvent SkaterSeated(Skater skater)
        {
            Contract.Requires(skater != null);
            return null;
        }

        public BoxEvent SkaterReleased(Skater skater)
        {
            Contract.Requires(skater != null);
            return null;
        }

        public bool VerifyMatch()
        {
            return false;
        }
    }
}
