﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Derbster.Domain;

namespace Derbster.Actions
{
    [ContractClass(typeof(MatchManagementActionsContract))]
    public interface IMatchManagementActions
    {
        Match CreateMatch(Roster firstRoster, Roster secondRoster);
        Match CreateMatch(string name,Roster firstRoster, Roster secondRoster);
        Match CreateRegulationMatch(Roster firstRoster, Roster secondRoster);
        Match CreateRegulationMatch(string name,Roster firstRoster, Roster secondRoster);
        Match CreateSanctionedMatch(Roster firstRoster, Roster secondRoster);
        Match CreateSanctionedMatch(string name, Roster firstRoster, Roster secondRoster);

        Match FindMatchById(string matchId);
        IEnumerable<Match> FindMatches(string keyword, bool matchWhole=false);
        IEnumerable<Match> FindMatchesByDate(DateTime from, DateTime to);
        IEnumerable<Match> FindMatchesByName(string name, bool matchWhole=false);
        IEnumerable<Match> FindMatchesByLocation(string location, bool matchWhole=false); 

        void RenameMatch(Match match, string name);
        void RenameMatch(string matchId, string name);

        void DeleteMatch(Match match);
        void DeleteMatch(string matchId);
    }

    [ContractClassFor(typeof(IMatchManagementActions))]
    abstract class MatchManagementActionsContract : IMatchManagementActions
    {
        public Match CreateMatch(Roster firstRoster, Roster secondRoster)
        {
            Contract.Requires(firstRoster != null);
            Contract.Requires(secondRoster != null);
            return null;
        }

        public Match CreateMatch(string name, Roster firstRoster, Roster secondRoster)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(firstRoster != null);
            Contract.Requires(secondRoster != null);
            return null;
        }

        public Match CreateRegulationMatch(Roster firstRoster, Roster secondRoster)
        {
            Contract.Requires(firstRoster != null);
            Contract.Requires(secondRoster != null);
            return null;
        }

        public Match CreateRegulationMatch(string name, Roster firstRoster, Roster secondRoster)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(firstRoster != null);
            Contract.Requires(secondRoster != null);
            return null;
        }

        public Match CreateSanctionedMatch(Roster firstRoster, Roster secondRoster)
        {
            Contract.Requires(firstRoster != null);
            Contract.Requires(secondRoster != null);
            return null;
        }

        public Match CreateSanctionedMatch(string name, Roster firstRoster, Roster secondRoster)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(firstRoster != null);
            Contract.Requires(secondRoster != null);
            return null;
        }

        public Match FindMatchById(string matchId)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(matchId));
            return null;
        }

        public IEnumerable<Match> FindMatches(string keyword, bool matchWhole = false)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(keyword));
            return null;
        }

        public IEnumerable<Match> FindMatchesByDate(DateTime fromDate, DateTime toDate)
        {
            Contract.Requires(fromDate < toDate);
            return null;
        }

        public IEnumerable<Match> FindMatchesByName(string name, bool matchWhole = false)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            return null;
        }

        public IEnumerable<Match> FindMatchesByLocation(string location, bool matchWhole = false)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(location));
            return null;
        }

        public void RenameMatch(Match match, string name)
        {
            Contract.Requires(match != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
        }

        public void RenameMatch(string matchId, string name)
        {
            Contract.Requires(String.IsNullOrWhiteSpace(matchId));
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
        }

        public void DeleteMatch(Match match)
        {
            Contract.Requires(match != null);
        }

        public void DeleteMatch(string matchId)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(matchId));
        }
    }
}
