﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Derbster.Domain;

namespace Derbster.Actions
{
    [ContractClass(typeof(PersonManagementActionsContract))]
    public interface IPersonManagementActions
    {
        Skater CreateSkater(string name, string number);
        Skater CreateSkater(string name, string number, Team team);

        Team CreateTeam(string name);
        Team CreateTeam(string name, League league);
        League CreateLeague(string name);
        Official CreateOfficial(string name);
        Official CreateOfficial(string name, string number);
        Official CreateOfficial(string name, League league);
        Official CreateOfficial(string name, string number, League league);

        void ChangeSkaterName(Skater skater, string name);
        void ChangeSkaterNumber(Skater skater, string number);
        void AddSkaterToTeam(Skater skater, Team team);
        void RemoveSkaterFromTeam(Skater skater, Team team);

        void ChangeTeamName(Team team, string name);

        void ChangeLeagueName(League league, string name);

        void DeleteOfficial(Official official);
        void DeleteSkater(Skater skater);
        void DeleteTeam(Team team);
        void DeleteLeague(League league);
    }

    [ContractClassFor(typeof(IPersonManagementActions))]
    abstract class PersonManagementActionsContract : IPersonManagementActions
    {
        public Skater CreateSkater(string name, string number)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(!String.IsNullOrWhiteSpace(number));
            Contract.Requires(number.Length > 0 && number.Length <= 4);

            return null;
        }

        public Skater CreateSkater(string name, string number, Team team)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(!String.IsNullOrWhiteSpace(number));
            Contract.Requires(number.Length > 0 && number.Length <= 4);
            Contract.Requires(team != null);

            return null;
        }

        public Team CreateTeam(string name)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            return null;
        }

        public Team CreateTeam(string name, League league)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(league != null);
            return null;
        }

        public League CreateLeague(string name)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            return null;
        }

        public Official CreateOfficial(string name)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            return null;
        }

        public Official CreateOfficial(string name, string number)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(!String.IsNullOrWhiteSpace(number));
            Contract.Requires(number.Length > 0 && number.Length <= 4);
            return null;
        }

        public Official CreateOfficial(string name, League league)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(league != null);
            return null;
        }

        public Official CreateOfficial(string name, string number, League league)
        {
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
            Contract.Requires(!String.IsNullOrWhiteSpace(number));
            Contract.Requires(number.Length > 0 && number.Length <= 4);
            Contract.Requires(league != null);

            return null;
        }

        public void ChangeSkaterName(Skater skater, string name)
        {
            Contract.Requires(skater != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
        }

        public void ChangeSkaterNumber(Skater skater, string number)
        {
            Contract.Requires(skater != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(number));
            Contract.Requires(number.Length > 0 && number.Length <= 4);
        }

        public void AddSkaterToTeam(Skater skater, Team team)
        {
            Contract.Requires(skater != null);
            Contract.Requires(team != null);
        }

        public void RemoveSkaterFromTeam(Skater skater, Team team)
        {
            Contract.Requires(skater != null);
            Contract.Requires(team != null);
        }

        public void ChangeTeamName(Team team, string name)
        {
            Contract.Requires(team != null);
            Contract.Requires(name != null);
        }

        public void ChangeLeagueName(League league, string name)
        {
            Contract.Requires(league != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(name));
        }
   
        public void DeleteOfficial(Official official)
        {
            Contract.Requires(official != null);
        }

        public void DeleteSkater(Skater skater)
        {
            Contract.Requires(skater != null);
        }

        public void DeleteTeam(Team team)
        {
            Contract.Requires(team != null);
        }
    
        public void DeleteLeague(League league)
        {
            Contract.Requires(league != null);
        }
    }
}
