﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Derbster.Actions;
using Derbster.DataAccess;

namespace Derbster.Console
{
    class Program
    {
        private static GameActionStack _actionStack;

        static void Main(string[] args)
        {
            using (var dbContext = new DerbsterContext())
            {
                _actionStack = new GameActionStack(dbContext);

                Stopwatch timer = new Stopwatch();
                timer.Start();

                var simulator = new GameGenerator(_actionStack, dbContext, .10);
                var match = simulator.SimulateMatch();

                if (!match.VerifyMatch()) System.Console.WriteLine("Verification Failed");

                timer.Stop();
                System.Console.WriteLine("Timer: " + timer.ElapsedMilliseconds);

                timer.Restart();
                using (var newContext = new DerbsterContext())
                {
                    var loadedMatch =
                        newContext.KnownMatches.Include(m => m.Jams).SingleOrDefault(m => m.MatchId == match.MatchId);

                    if (loadedMatch == null || !loadedMatch.VerifyMatch()) System.Console.WriteLine("Verification Failed");
                }
                timer.Stop();
                System.Console.WriteLine("Timer: " + timer.ElapsedMilliseconds);
                System.Console.ReadLine();
            }
        }
    }
}
