﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Derbster.Domain;

namespace Derbster.DataAccess
{
    public class DerbsterContext : DbContext, IDerbsterContext
    {
        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            Database.SetInitializer(new CreateDatabaseIfNotExists<DerbsterContext>());
  
            modelBuilder.Entity<BoxEvent>().HasRequired(be => be.Skater).WithMany();

            modelBuilder.Entity<League>().HasMany(l => l.Teams).WithOptional(t => t.League);

            modelBuilder.Entity<Lineup>().HasOptional(l => l.Jammer).WithMany();
            modelBuilder.Entity<Lineup>().HasOptional(l => l.Pivot).WithMany();
            modelBuilder.Entity<Lineup>().HasMany(l => l.Blockers).WithMany();

            modelBuilder.Entity<Match>().HasRequired(m => m.FirstTeamRoster).WithOptional();
            modelBuilder.Entity<Match>().HasRequired(m => m.SecondTeamRoster).WithOptional();
            modelBuilder.Entity<Match>().HasMany(m => m.Jams).WithRequired(j => j.Match);
            modelBuilder.Entity<Match>().HasOptional(m => m.OfficialRoster).WithOptionalPrincipal(m => m.Match);
            modelBuilder.Entity<Match>().HasMany(m => m.Penalties).WithRequired(p=>p.Match);
            modelBuilder.Entity<Match>().HasMany(m => m.BoxEvents).WithRequired(p=>p.Match);

            modelBuilder.Entity<Official>().HasOptional(o => o.League).WithMany(l => l.Officials);

            modelBuilder.Entity<Roster>().HasMany(r => r.Members).WithMany(s=>s.Rosters);

            modelBuilder.Entity<Scoring>().HasRequired(s => s.Jammer).WithMany();
            modelBuilder.Entity<Scoring>().HasMany(s => s.Passes).WithRequired(p=>p.Scoring);

            modelBuilder.Entity<Skater>().HasMany(s => s.Teams).WithMany(t => t.Skaters);
        }

        public IDbSet<Match> KnownMatches { get; set; }
        public IDbSet<League> KnownLeagues { get; set; }
        public IDbSet<Official> KnownOfficials { get; set; }
        public IDbSet<Roster> KnownRosters { get; set; }
        public IDbSet<Skater> KnownSkaters { get; set; }
        public IDbSet<Team> KnownTeams { get; set; }
        
    }
}
