﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace Derbster.Domain
{
    public enum LineupPosition
    {
        Blocker,
        Pivot,
        Jammer
    }
    public class Lineup : IEnumerable<Skater>
    {
        [ContractVerification(false)]
        internal Lineup()
        {
            
        }

        public Lineup(HashSet<Skater> blockers)
        {
            Contract.Requires(blockers != null && blockers.Count > 0 && blockers.Count <= 5);
            Blockers = blockers;
            Pivot = null;
            Jammer = null;
        }

        public Lineup(Skater jammer, HashSet<Skater> blockers)
        {
            Contract.Requires(jammer != null);
            Contract.Requires(blockers != null && blockers.Count <= 4);
            Blockers = blockers;
            Pivot = null;
            Jammer = jammer;
        }

        public Lineup(Skater jammer, Skater pivot, HashSet<Skater> blockers)
        {
            Contract.Requires(jammer != null);
            Contract.Requires(pivot != null);
            Contract.Requires(blockers != null && blockers.Count <= 3);

            Blockers = blockers;
            Jammer = jammer;
            Pivot = pivot;
        }

        public Skater Pivot { get; set; }
        public HashSet<Skater> Blockers { get; set; }
        public Skater Jammer { get; set; }
        public bool StarPassed { get; set; }

        [Key, DatabaseGenerated(DatabaseGeneratedOption.Identity)]
        public int LineupId {get; set;}

        public IEnumerator<Skater> GetEnumerator()
        {
            if (Jammer != null) yield return Jammer;
            if (Pivot != null) yield return Pivot;
            foreach (var blocker in Blockers)
            {
                yield return blocker;
            }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
    }
}
