﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace Derbster.Domain
{
    public enum OfficialPosition
    {
        HeadReferee,
        InsidePackReferee,
        OutsidePackReferee,
        JamReferee,
        JamTimer,
        PenaltyBoxManager,
        PenaltyBox,
        PenaltyWrangler,
        PenaltyTracker,
        ScoreKeeper,
        ScoreBoard,
        Lineup,
    }

    public class Official
    {
        [ContractVerification(false)]
        internal Official()
        {
            
        }
        public Official(string name)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(name));
            Name = name;
        }

        public Official(string name, string number)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(name));
            Contract.Requires(!string.IsNullOrWhiteSpace(number));
            Contract.Requires(number.Length > 0 && number.Length <= 4);
            Name = name;
            Number = number;
        }
        [Key]
        public string OfficialId { get; set; }
        public string Name { get; set; }
        public string Number { get; set; }

        public League League { get; set; }
        public virtual HashSet<Match> Matches { get; set; }

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(!string.IsNullOrWhiteSpace(Name));
            Contract.Invariant(Number==null || (Number.Length > 0 && Number.Length <= 4));
        }
    }
}
