﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace Derbster.Domain
{
    public class Penalty : IEquatable<Penalty>
    {
        [ContractVerification(false)]
        internal Penalty()
        {
            
        }

        public Penalty(Skater skater, string code, int period, int jamNumber)
        {
            Contract.Requires(skater != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(code));
            Contract.Requires(period == 1 || period == 2);
            Contract.Requires(jamNumber >= 1);
            Code = code;
            Period = period;
            Skater = skater;
            JamNumber = jamNumber;
        }

        [Key, DatabaseGenerated(DatabaseGeneratedOption.Identity)]
        public int PenaltyId { get; set; }

        public Match Match { get; set; }
        public int Period { get; set; }
        public int JamNumber { get; set; }
        public Skater Skater { get; set; }
        public string Code { get; set; }

        public bool Equals(Penalty other)
        {
            if (other == null) return false;
            if (PenaltyId == other.PenaltyId) return true;

            return Skater == other.Skater && Period == other.Period && 
                JamNumber == other.JamNumber && Code ==other.Code;
        }
    }
}
