﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace Derbster.Domain
{
    public class Roster : IEnumerable<Skater>
    {
        [ContractVerification(false)]
        internal Roster()
        {
            
        }

        public Roster(string name, string color)
        {
            Contract.Requires(name != null);
            Contract.Requires(color != null);
            Contract.Ensures(Members != null);

            Color = color;
            Name = name;

            Members = new HashSet<Skater>();
        }
  
        public string Color { get; set; }
        public string Name { get; set; }

        public Skater Captain { get; set; }
        public Skater Alternate { get; set; }
        
        public HashSet<Skater> Members { get; set; }

        [Key, DatabaseGenerated(DatabaseGeneratedOption.Identity)]
        public int RosterId {get; set;}

        public IEnumerator<Skater> GetEnumerator()
        {
            Contract.Assume(Members != null);
            return Members.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
    }
}
