﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace Derbster.Domain
{
    public enum SkaterState
    {
        Ready,
        InQueue,
        InBox,
        FouledOut,
        Ejected 
    }

    public enum SkaterPosition
    {
        Blocker,
        Jammer,
        Pivot
    }

    public class Skater : IEquatable<Skater>
    {
        [ContractVerification(false)]
        internal Skater()
        {
            
        }

        public Skater(string name, string number)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(name));
            Contract.Requires(!string.IsNullOrWhiteSpace(number) && number.Length <= 4);
            Name = name;
            Number = number;
            State = SkaterState.Ready;
        }

        [Key]
        public string SkaterId { get; set; }
        public string Name { get; set; }
        public string Number { get; set; }
        public SkaterState State { get; set; }

        public virtual HashSet<Team> Teams { get; set; }
        public virtual HashSet<Roster> Rosters { get; set; }

        public bool Equals(Skater other)
        {
            Contract.Assume(other != null);
            return SkaterId.Equals(other.SkaterId);
        }
    }
}
